/***************************************************************************
 *   Copyright (C) 2004 by Stephen Anthony                                 *
 *   stephena@users.sourceforge.net                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef KSTELLA_H
#define KSTELLA_H

#include <kapplication.h>
#include <kmainwindow.h>
#include <kaction.h>

#include <qwidget.h>
#include <qsplitter.h>
#include <qmenubar.h>

#include <gamelistdoc.h>
#include <gamelistview.h>
#include <previewwindow.h>

class Game;

/** The base class for the KStella application. It sets up the main window and
	reads the config file as well as providing a menubar.

	An GameListView instance creates the game listing, and a PreviewWindow instance
	shows the snapshots, labels and manuals.

	@author Stephen Anthony
*/
class KStella : public KMainWindow
{
  Q_OBJECT

	public:
		/** constructor of KStella, calls all init functions to create the application */
		KStella( void );
		~KStella( void );

		/** returns the ROM view */
		static int romView( void ) { return _romView; }

		/** What ROMS does the user want to see */
		enum viewRoms { ALL, AVAILABLE, UNAVAILABLE };

	private:
		/** sets the ROM view */
		static void setRomView( int romView ) { _romView = romView; }

	 	/** save general Options like all bar positions and status as well as the
			geometry and the recent file list to the configuration file */
		void saveOptions( void );

		/** creates the menubar and toolbar and actions */
		void initActions( void );

		/** creates the main widgets of the program */
		void initView( void );

		/** shows the preview window based on the given flag */
		void showPreview( bool toggle );

	private slots:
		/** closes all open windows */
		void slotFileQuit( void );

		/** update the Stella emulator settings */
		void slotEmulatorDefaults( void );

		/** update the Stella game defaults */
		void slotGameDefaults( void );

		/** reload the Stella roms */
		void slotEmulatorReload( void );

		/** sets the view to All ROMS */
		void slotViewAllRoms( void );

		/** sets the view to Available ROMS */
		void slotViewAvailableRoms( void );

		/** sets the view to Unavailable ROMS */
		void slotViewUnavailableRoms( void );

		/** toggles viewing of rarity */
		void slotToggleRarity( void );

		/** toggles viewing of manufacturer */
		void slotToggleManufacturer( void );

		/** toggles viewing of the game info as each game is selected */
		void slotToggleGameInfo( void );

		/** toggles viewing of the preview window */
		void slotTogglePreview( void );

		/** changes the settings for the current game */
		void slotSettings( void );

		/** start emulation for the current game */
		void slotEmulate( void );

		/** shows some stats for the current properties file */
		void slotShowStatistics( void );

		/** shows the Stella documentation, if it's installed */
		void slotShowStellaDocumentation( void );

		/** shows standard key configuration dialog */
		void slotConfigureKeys( void );

		/** update status bar with game notes when a new game is selected */
		void slotChangeNote( Game* g );

	private:
		/** the configuration object of the application */
		KConfig* config;

		/** this is responsible for loading all game roms in the system */
		GameListDoc* doc;

		/** the widget responsible for showing a list of games in a list box */
		GameListView* view;

		/** the widget responsible for showing snapshots and labels */
		PreviewWindow* preview;

		/** separates the game listing and preview window */
		QSplitter* splitter;

		/** does the user want to see ALL, AVAILABLE or UNAVAILABLE roms */
		static int _romView;

		/** KAction menu items */
		KToggleAction *actionAllRoms, *actionAvailableRoms, *actionUnavailableRoms;
		KToggleAction *actionRarity, *actionManufacturer;
		KToggleAction *actionPreviewWindow, *actionDebugWindow;
};

#endif
